from typing import Collection

import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns

from . import clust_compute as compute


def hist_external(
    adata,
    partition_keys: Collection[str],
    obsm_representation: Collection[str] = None,
):
    metrics_dict = dict(
        Davies_Bouldin=compute.davies_bouldin,
        silouhette=compute.silhouette,
        calinski=compute.calinski_harabasz,
    )
    # ,
    # DBCV=compute.dbcv)  # Import dbcv...
    # f = plt.figure()
    i = 1
    for name, func in metrics_dict.items():
        plt.subplot(1, len(metrics_dict), i)
        to_plot_array = []
        for key in partition_keys:
            score = func(adata, partition_key=key, obsm_representation=None)
            method_representation = f"{key}_original"
            metric = name
            to_plot_array.append([score, method_representation, metric])
            if obsm_representation:
                for rep in obsm_representation:
                    score = func(
                        adata, partition_key=key, obsm_representation=rep
                    )
                    method_representation = f"{key}_{rep}"
                    metric = name
                    to_plot_array.append(
                        [score, method_representation, metric]
                    )
        to_plot_df = pd.DataFrame(
            to_plot_array,
            columns=["score", "clustering_representation", "metric"],
        )
        sns.barplot(
            x="metric",
            y="score",
            hue="clustering_representation",
            ax=plt.gca(),
            data=to_plot_df,
        )
        i += 1


def hist_internal(adata, partition_keys: Collection[str], reference):
    metrics_dict = dict(
        rand=compute.rand,
        FowlkesMallows=compute.fowlkes_mallows,
        NMI=compute.nmi,
        vmeasure=compute.vmeasure,
    )
    # f = plt.figure()
    to_plot_array = []
    i = 0
    for key in partition_keys:
        for name, func in metrics_dict.items():
            score = func(adata, key, reference)
            clustering = key
            metric = name
            to_plot_array.append([score, clustering, metric])
            i += 1
    to_plot_df = pd.DataFrame(
        to_plot_array, columns=["score", "clustering", "metric"]
    )
    out = sns.barplot(x="metric", y="score", hue="clustering", data=to_plot_df)
    return out



"""
Created on Thu Jun  4 16:38:47 2020

@author: antoinecollin
"""

from typing import Collection, Optional

import anndata
import numpy as np
import pandas as pd
from get_data import get_average_celltype_counts
from scipy.stats import entropy


def one_v_all_matrix(adata: anndata, partition_key: str = "CellType"):
    """
    Computes the one_v_all specificity of all genes for all celltype.
    It is defined as the average expression of a gene in a celltype
    divided by the sum of mean expressions in every celltypes.

    Parameters
    ----------
    adata : anndata
        Annotated data matrix.

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    spe_one_v_all : DataFrame
        DataFrame of shape n_genes x n_celltype containing the one_v_all
        specificities of all genes by celltypes

    """
    average_by_celltype = get_average_celltype_counts(
        adata=adata, partition_key=partition_key
    )
    spe_matrix = pd.DataFrame(
        columns=list(average_by_celltype.columns),
        index=list(average_by_celltype.index),
    )
    for celltype in average_by_celltype.index:
        spe_matrix.loc[celltype] = average_by_celltype.loc[
            celltype
        ] / average_by_celltype.sum(axis=0)
    return spe_matrix


def one_v_max_matrix(adata: anndata, partition_key: str = "CellType"):
    """
    Computes the one_v_max specifity of all genes for the celltypes which
    maximizes it. For a gene, one_v_max specificity is defined as the ratio
    between the average expression of this gene in the most expressed
    celltype divided by the second most expressed celltype.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with the
        value of the specificity and the specific celltype name.

    """
    try:
        adata.uns[f"ovm_{partition_key}"]
    except KeyError:
        average_by_celltype = get_average_celltype_counts(
            adata=adata, partition_key=partition_key
        )
        spe_matrix = pd.DataFrame(
            columns=list(average_by_celltype.columns),
            index=list(average_by_celltype.index),
        )
        for celltype in average_by_celltype.index:
            spe_matrix.loc[celltype] = average_by_celltype.loc[
                celltype
            ] / average_by_celltype.loc[
                average_by_celltype.index != celltype
            ].max(
                axis=0
            )
        adata.uns[f"ovm_{partition_key}"] = spe_matrix
    else:
        spe_matrix = adata.uns[f"ovm_{partition_key}"]
    return spe_matrix


def shannon_average(adata: anndata, partition_key: str = "CellType"):
    """
    Computes the Shannon entropy of every gene based on the distribution of
    their average value across celltypes.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with the
        value of the specificity and the specific celltype name.

    """
    prob_matrix = one_v_all_matrix(adata=adata, partition_key=partition_key)
    spe_shannon = pd.Series(
        index=prob_matrix.columns, name="Shannon_entropies"
    )
    for gene in prob_matrix.columns:
        spe_shannon[gene] = entropy(list(prob_matrix[gene]))
    return spe_shannon


# TODO
def shannon_sample(adata: anndata, genes: Optional[Collection[str]]):
    """
    Computes the Shannon entropy of the gene distribution across the
    celltypes based on a subsample of the expression matrix. The sampling
    has a normalizing effect, preventing the most represented celltypes
    to overshadow the rare ones.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    genes : Collection[str]
        List of genes for which to compute the specificities. If not
        specified, it is computed for all the genes.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with
        the value of the specificity and the specific celltype name.


    """


def tau_average(adata: anndata, partition_key: str = "CellType"):
    """
    Computes tau coefficient of every gene based on the distribution of
    their average value across celltypes.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with the
        value of the specificity and the specific celltype name.

    """
    average_by_celltype = get_average_celltype_counts(
        adata=adata, partition_key=partition_key
    )
    ratio_matrix = pd.DataFrame(
        columns=list(average_by_celltype.columns),
        index=list(average_by_celltype.index),
    )
    for celltype in average_by_celltype.index:
        ratio_matrix.loc[celltype] = average_by_celltype.loc[
            celltype
        ] / average_by_celltype.max(axis=0)
    inverted_ratio = 1 - ratio_matrix
    return inverted_ratio.sum(axis=0) / (len(average_by_celltype.index) - 1)


# TODO
def tau_sample(adata: anndata, genes: Optional[Collection[str]]):
    """

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    genes : Collection[str]
        List of genes for which to compute the specificities. If not
        specified, it is computed for all the genes.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with
        the value of the specificity and the specific celltype name.

    """


def gini_average(adata: anndata, partition_key: str = "CellType"):
    """
    Computes gini coefficient of every gene based on the distribution of
    their average value across celltypes.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with
        the value of the specificity and the specific celltype name.

    """
    average_by_celltype = get_average_celltype_counts(
        adata=adata, partition_key=partition_key
    )
    spe_gini = pd.Series(
        index=average_by_celltype.columns, name="gini_Coefficient"
    )
    n = average_by_celltype.shape[0]
    index = np.arange(1, n + 1)
    for gene in average_by_celltype.columns:
        gini_list = average_by_celltype.loc[:, gene].sort_values()
        spe_gini[gene] = (np.sum((2 * index - n - 1) * gini_list)) / (
            n * np.sum(gini_list)
        )
    return spe_gini


# TODO
def gini_sample(adata: anndata, genes: Optional[Collection[str]]):
    """

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    genes : Collection[str]
        List of genes for which to compute the specificities. If not
        specified, it is computed for all the genes.

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with
        the value of the specificity and the specific celltype name.

    """


# TODO
def dist_gene_to_celltype(adata):
    """
    Just from the adata, identifies the marker genes and corresponding
    celltypes. Those are computed using the criteria of the fonction
    (threshold, metric etc...)

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    Returns
    -------
    spe_one_v_max : dict
        Dictionary where keys are genes and attributes are lists with the
        value of the specificity and the specific celltype name.

    """


# To Move To Analysis


# if __name__ == "__main__" :
#     a = specificity_summary(adata=barbry,marker_genes =
#     mark_dict,partition_key='CellType')



"""
Created on Thu Jun  4 16:38:47 2020

@author: antoinecollin
"""

import datetime
import os
from pathlib import Path

import anndata
import compute
import matplotlib.pyplot as plt
import pandas as pd
import plot
from get_data import get_average_celltype_counts, get_spe


def specificity_summary(adata, marker_genes, partition_key: str = "CellType"):
    """
    Returns a summary of the specificity metrics for the dataset.

    Parameters
    ----------
    adata
        The corrected expression matrix

    marker_genes
        A dictionary containing the celltypes and their marker genes to analyse
        in the following format {celltype1 : [gene1,gene2,...],celltype2 : ...}

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    Returns
    -------
    summary
        The summary as a pandas DataFrame with the following columns :
        gene : name of the gene
        expected celltype : supposedly marked celltype by this gene
        most expressed celltype : celltype in which the gene is most expressed
        one_v_max,shannon,tau,gini : specificity values for this gene

    """
    shannon = get_spe(adata, spe_metric="shannon", partition_key=partition_key)
    tau = get_spe(adata, spe_metric="tau", partition_key=partition_key)
    gini = get_spe(adata, spe_metric="gini", partition_key=partition_key)
    one_v_max = compute.one_v_max_matrix(adata, partition_key=partition_key)
    summary = pd.DataFrame(
        columns=[
            "gene",
            "expected celltype",
            "most expressed celltype",
            "one_v_max",
            "shannon",
            "tau",
            "gini",
        ]
    )

    for celltype, genes in marker_genes.items():
        for gene in genes:
            if (
                gene in shannon.keys()
            ):  # Check why some of them return an error
                summary.loc[gene, :] = [
                    gene,
                    celltype,
                    pd.to_numeric(one_v_max[gene]).idxmax(),
                    max(one_v_max[gene]),
                    shannon[gene],
                    tau[gene],
                    gini[gene],
                ]
    return summary


# TODO
def specificity_quality_control(
    adata: anndata,
    marker_genes: dict,
    partition_key: str,
    project_dir: str,
):
    """
    Performs an analysis of the identified marker genes with regard to
    specificity and save the files of the analysis.

    Parameters
    ----------
    adata
        Annotated expression matrix

    marker_genes
        Marker genes dict to analyze

    partition_key
        The key in adata.obs corresponding to the annotations to be used.

    project_dir
        Path of the project where the plots should be saved


    Returns
    -------
    For every celltypes, plot and save 2 figures :
    -The distribution of the marker genes supported by their shannon, gini,
    au specificity
    -The one_v_max specificity repartition of the celltype for which the marker
     genes are most specific.

    Also computes and save a summary of the analysis

    """
    celltypes = list(marker_genes.keys())
    celltypes_checks = list(
        get_average_celltype_counts(adata, partition_key=partition_key).index
    )
    if sorted(celltypes) != sorted(celltypes_checks):
        print("Celltypes from anndata and marker genes file don't match")
        celltypes = list(set(celltypes) & set(celltypes_checks))
        marker_genes = {keys: marker_genes[keys] for keys in celltypes}
    now = datetime.datetime.now()
    project_dir_path = Path(project_dir)
    if not project_dir_path.joinpath(Path("SpecAnalysis")).is_dir():
        os.mkdir(project_dir_path.joinpath("SpecAnalysis"))
    os.mkdir(
        project_dir_path.joinpath(
            f'SpecAnalysis/Analysis_{now.strftime("%y%m%d_%H%M")}'
        )
    )
    analysis_path = project_dir_path.joinpath(
        f'SpecAnalysis/Analysis_{now.strftime("%y%m%d_%H%M")}'
    )
    for celltype in celltypes:
        cell_path = analysis_path.joinpath(f"{celltype}")
        os.mkdir(cell_path)
        gene_list = list(set(marker_genes[celltype]) & set(adata.var.index))
        # in case some gene names are different/don't exist between
        # the marker gene file and adata file
        plot.marker_genes_distribution(
            adata=adata,
            gene_list=gene_list,
            celltype=celltype,
            partition_key=partition_key,
        )
        plt.savefig(cell_path.joinpath(f"gene_distribution_{celltype}"))
        plt.close()
        plot.one_v_max_genelist(
            adata=adata, gene_list=gene_list, partition_key=partition_key
        )
        plt.savefig(cell_path.joinpath(f"one_v_max_for_{celltype}_genes"))
        plt.close()
    summary = specificity_summary(
        adata=adata, marker_genes=marker_genes, partition_key=partition_key
    )
    summary.to_csv(
        path_or_buf=analysis_path.joinpath("spec_summary.tsv"), sep="\t"
    )
    plot.all_spec_distrib(adata=adata, partition_key=partition_key)
    plt.savefig(analysis_path.joinpath("spec_distrib.png"))
    plt.close()


# if __name__ == '__main__':
#     specificity_quality_control(adata=barbry,
#                                 marker_genes=marker_dict,
#                                 partition_key='CellType',
#                                 project_dir = r'
#                                 C:\Users\ipmc\PycharmProjects\test&tuto\Kobaye')

# Saves in an ad hoc 'result' directory :
# - gene_distribs, one for each celltype
# - one_v_max distrib : one_v_max distrib in the celltypes. Markers of
# different celltypes marked with different colors ? --> messy.
# For each gene, just the stripplot
# one_v_max in the most expressed celltype. (with legend pex 'FOXJ1,
# max in 'Basal')
# - an excel file with gene | expected celltype | most expressed celltype |
# one_v_max | Shannon | Tau | Gini


# adata = barbry
# genes = dict({celltype : [Gene1,Gene2,...]}. On gerera les
# autres types de fichier
# plus tard


# TODO
def specificity_gene_selection(adata: anndata, many, other, arguments):
    """

    Parameters
    ----------
    adata
    many
    other
    arguments

    Returns
    -------

    """
    return "do_nothing"



from time import sleep

from dask import delayed


def inc(x):
    sleep(1)
    return x + 1


def add(x, y):
    sleep(1)
    return x + y


def working():
    x = delayed(inc)(1)
    y = delayed(inc)(2)
    z = delayed(add)(x, y)
    z.compute()


if __name__ == "__main__":
    print("Dask tuto")
    # now = datetime.now().time()  # time object
    # print("now =", now)
    # client = Client()
    # working()
    # client.close()
    # now = datetime.now().time()  # time object
    # print("now =", now)



import csv
import inspect
import os
import webbrowser

import anndata
import matplotlib
import scanpy as sc
from dask.distributed import Client, LocalCluster, wait

from . import atlas, folders, multiqc

# try:
#     from . import atlas, folders, multiqc
# except ImportError:
#     import atlas
#     import folders
#     import multiqc


"""
checkatlas base module.
This is the principal module of the checkatlas project.

"""

EXTENSIONS = [".h5ad", ".h5"]
# EXTENSIONS = [".rds", ".h5ad", ".h5"]
CELLRANGER_FILE = "/outs/filtered_feature_bc_matrix.h5"
RSCRIPT = inspect.getfile(atlas).replace("atlas.py", "convertSeurat.R")
SUMMARY_EXTENSION = "_checkatlas_summ.tsv"
ADATA_EXTENSION = "_checkatlas_adata.tsv"
QC_EXTENSION = "_checkatlas_qc.png"
UMAP_EXTENSION = "_checkatlas_umap.png"
TSNE_EXTENSION = "_checkatlas_tsne.png"
METRIC_CLUSTER_EXTENSION = "_checkatlas_mcluster.tsv"
METRIC_ANNOTATION_EXTENSION = "_checkatlas_mannot.tsv"
METRIC_DIMRED_EXTENSION = "_checkatlas_mdimred.tsv"
METRIC_SPECIFICITY_EXTENSION = "_checkatlas_mspecificity.tsv"


def list_atlases(path) -> list:
    """
    List all atlases files in the path
    Detect .rds, .h5, .h5ad
    :param path:
    :return: List of files
    """
    atlas_list = list()
    for root, dirs, files in os.walk(path):
        for file in files:
            for extension in EXTENSIONS:
                if file.endswith(extension):
                    atlas_list.append(os.path.join(root, file))
    return atlas_list


def get_atlas_name(atlas_path):
    """
    From atlas_path extract the atlas_name
    :param atlas_path:
    :return:
    """
    return os.path.splitext(os.path.basename(atlas_path))[0]


def get_atlas_extension(atlas_path):
    """
    From atlas_path extract the atlas file extension
    :param atlas_path:
    :return:
    """
    return os.path.splitext(os.path.basename(atlas_path))[1]


def convert_seurat_atlas(atlas_path, atlas_name) -> bool:
    """
    Convert all Seurat atlas to Scanpy
    :param path:
    :param atlas_list:
    :return:
    """
    atlas.convert_atlas(atlas_path, atlas_name)
    return True


def clean_list_atlases(atlas_list, path) -> dict:
    """
    Go through all files and detect Seurat or Scanpy Atlas
    Then:
    - Convert Seurat files to Scanpy
    - Clean Scanpy files
    :param atlas_list:
    :return: clean_list_atlas will only cleaned Scanpy atlas. A dict with
    these info ['Atlas_Name','Type','Atlas_file_extension',
    'Checkatlas_folder']
    """
    # Create dict with these info ['Atlas_Name','Type','Converted',
    # 'Checkatlas_path']
    clean_atlas_dict = dict()
    for atlas_path in atlas_list:
        atlas_name = get_atlas_name(atlas_path)
        if atlas_path.endswith(".rds"):
            atlas_h5 = atlas_path.replace(".rds", ".h5ad")
            if os.path.exists(atlas_h5):
                print("Seurat file already converted to Scanpy:", atlas_h5)
            else:
                convert_seurat_atlas(atlas_path, atlas_name)
                if os.path.exists(atlas_h5):
                    print(
                        "Include Atlas: " + atlas_name + " from " + atlas_path
                    )
                    info = [
                        atlas_name,
                        "Seurat",
                        ".rds",
                        os.path.dirname(atlas_path) + "/",
                    ]
                    clean_atlas_dict[atlas_h5] = info
        elif atlas_path.endswith(".h5"):
            # detect if its a cellranger output
            if atlas_path.endswith(CELLRANGER_FILE):
                atlas_h5 = atlas_path.replace(CELLRANGER_FILE, "")
                atlas_name = get_atlas_name(atlas_h5)
                print("Include Atlas: " + atlas_name + " from " + atlas_path)
                info = [
                    atlas_name,
                    "Cellranger",
                    ".h5",
                    os.path.dirname(atlas_h5) + "/",
                ]
                clean_atlas_dict[atlas_path] = info
        elif atlas_path.endswith(".h5ad"):
            atlas_rds = atlas_path.replace(".h5ad", ".rds")
            if os.path.exists(atlas_rds):
                print("Include Atlas: " + atlas_name + " from " + atlas_path)
                info = [
                    atlas_name,
                    "Seurat",
                    ".rds",
                    os.path.dirname(atlas_path) + "/",
                ]
                clean_atlas_dict[atlas_path] = info
            else:
                print("Include Atlas: " + atlas_name + " from " + atlas_path)
                info = [
                    atlas_name,
                    "Scanpy",
                    ".h5ad",
                    os.path.dirname(atlas_path) + "/",
                ]
                clean_atlas_dict[atlas_path] = info
    # open file for writing, "w" is writing
    dict_file = open(path + "/checkatlas_files/list_atlases.csv", "w")
    w = csv.writer(dict_file)
    # loop over dictionary keys and values
    for key, val in clean_atlas_dict.items():
        # write every key and value to file
        w.writerow([key, ",".join(val)])
    dict_file.close()
    return clean_atlas_dict


def read_atlas(atlas_path, atlas_info):
    print("--- Load " + atlas_info[0] + " in " + atlas_info[-1])
    try:
        if atlas_path.endswith(".h5"):
            print(atlas_path)
            adata = sc.read_10x_h5(atlas_path)
            adata.var_names_make_unique()
        else:
            adata = sc.read_h5ad(atlas_path)
        return adata
    except anndata._io.utils.AnnDataReadError:
        print("AnnDataReadError, cannot read:", atlas_info[0])
        return None


def summary_table(adata, atlas_path, atlas_name, resume) -> None:
    """
    Extract summary tables from the adata
    :param adata:
    :param atlas_path:
    :param atlas_name:
    :param resume: True if you want to run the function only if the tables
     does not exist already
    :return:
    """
    # create summary table
    csv_path = atlas_path.replace(".h5ad", SUMMARY_EXTENSION)
    print(csv_path)
    if resume:
        if not os.path.isfile(csv_path):
            atlas.create_summary_table(adata, csv_path, atlas_name, path)
    else:
        atlas.create_summary_table(adata, csv_path, atlas_name, path)

    # Create AnnData table
    csv_path = atlas_path.replace(".h5ad", ADATA_EXTENSION)
    if resume:
        if not os.path.isfile(csv_path):
            atlas.create_anndata_table(adata, csv_path, atlas_name, path)
    else:
        atlas.create_anndata_table(adata, csv_path, atlas_name, path)


def start_multithread_client():
    # setup the cluster
    cluster = LocalCluster()
    client = Client(cluster)
    print("Opening Dask browser: http://localhost:8787/status")
    webbrowser.open("http://localhost:8787/status")
    return client


def run(path, atlas_list, multithread, n_cpus):
    """
    Main function of checkatlas
    Run all functions for all atlases:
    - Clean files list by getting only Scanpy atlas (converted from Seurat
    if necessary)
    - Extract summary tables
    - Create UMAP and T-sne figures
    - Calculate every metrics
    Dask powered -- everything is done in parallel

    :param path:
    :param atlas_list:
    :param n_cpus:
    :return:
    """

    folders.checkatlas_folders(path)

    if multithread:
        client = start_multithread_client()
        futures = list()

    # First clean atlas list and keep only the h5ad files
    clean_atlas_dict = clean_list_atlases(atlas_list, path)

    # Create summary files
    for atlas_path, atlas_info in clean_atlas_dict.items():
        if multithread:
            matplotlib.pyplot.switch_backend("Agg")
            # #### Multi-threaded
            # read adata
            atlas_name = atlas_info[0]
            future_name = "Read_" + atlas_name
            adata = client.submit(
                read_atlas, atlas_path, atlas_info, key=future_name
            )
            future_name = "Clean_" + atlas_name
            adata = client.submit(
                atlas.clean_scanpy_atlas, adata, atlas_info, key=future_name
            )
            # Create summary files
            future_name = "Summary_" + atlas_name
            future_sum = client.submit(
                atlas.create_summary_table,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_sum)
            # Adata explorer
            future_name = "Adata_" + atlas_name
            future_sum = client.submit(
                atlas.create_anndata_table,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_sum)
            # Create QC plots
            future_name = "QC_" + atlas_name
            future_sum = client.submit(
                atlas.create_qc_plots,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_sum)
            # Create Umap and TSNE figures
            future_name = "UMAP_" + get_atlas_name(atlas_path)
            future_fig_umap = client.submit(
                atlas.create_umap_fig,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_fig_umap)
            future_name = "TSNE_" + get_atlas_name(atlas_path)
            future_fig_tsne = client.submit(
                atlas.create_tsne_fig,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_fig_tsne)
            # Calculate metrics
            future_name = "Metric_Cluster_" + get_atlas_name(atlas_path)
            future_met = client.submit(
                atlas.metric_cluster,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name,
            )
            futures.append(future_met)
            future_name2 = "Metric_Annot_" + get_atlas_name(atlas_path)
            future_met2 = client.submit(
                atlas.metric_annot,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name2,
            )
            futures.append(future_met2)
            future_name3 = "Metric_DimRed_" + get_atlas_name(atlas_path)
            future_met3 = client.submit(
                atlas.metric_dimred,
                adata,
                atlas_path,
                atlas_info,
                path,
                key=future_name3,
            )
            futures.append(future_met3)
            # Wait for all thread to end
            wait(futures)
        else:
            # ##### Sequential
            atlas_name = atlas_info[0]
            csv_summary_path = os.path.join(
                folders.get_folder(path, folders.SUMMARY),
                atlas_name + SUMMARY_EXTENSION,
            )
            if not os.path.exists(csv_summary_path):
                adata = read_atlas(atlas_path, atlas_info)
                if adata is not None:
                    adata = atlas.clean_scanpy_atlas(adata, atlas_info)
                    atlas.create_summary_table(
                        adata, atlas_path, atlas_info, path
                    )
                    atlas.create_anndata_table(
                        adata, atlas_path, atlas_info, path
                    )
                    atlas.create_qc_plots(adata, atlas_path, atlas_info, path)
                    atlas.create_umap_fig(adata, atlas_path, atlas_info, path)
                    atlas.create_tsne_fig(adata, atlas_path, atlas_info, path)
                    atlas.metric_cluster(adata, atlas_path, atlas_info, path)
                    atlas.metric_annot(adata, atlas_path, atlas_info, path)
                    atlas.metric_dimred(adata, atlas_path, atlas_info, path)
            else:
                print("Checkatlas already ran for:", atlas_name)

    print("Run MultiQC")
    multiqc.run_multiqc(path)


if __name__ == "__main__":
    path = "/Users/christophebecavin/Documents/testatlas/"
    # atlas_path = "/Users/christophebecavin/Documents/testatlas/"
    # atlas_info = ["test_version", "Scanpy", ".h5ad",
    # "data/test_version.h5ad"]
    folders.checkatlas_folders(path)
    atlas_list = list_atlases(path)
    clean_atlas_dict = clean_list_atlases(atlas_list, path)

    for atlas_path, atlas_info in clean_atlas_dict.items():
        print(atlas_path, atlas_info)
        adata = read_atlas(atlas_path, atlas_info)
        print(adata is not None)
        if adata is not None:
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_summary_table(adata, atlas_path, atlas_info, path)
            atlas.create_anndata_table(adata, atlas_path, atlas_info, path)
            # atlas.create_qc_plots(adata, atlas_path, atlas_info, path)
            atlas.create_umap_fig(adata, atlas_path, atlas_info, path)
            # atlas.create_tsne_fig(adata, atlas_path, atlas_info, path)
            atlas.metric_cluster(adata, atlas_path, atlas_info, path)
            atlas.metric_annot(adata, atlas_path, atlas_info, path)
            atlas.metric_dimred(adata, atlas_path, atlas_info, path)

# atlas_path = '/Users/christophebecavin/Documents/testatlas/hca/
# HCA_Barbry_Grch38_Raw_filter_Norm.h5ad'
# #atlas_path = '/Users/christophebecavin/Documents/testatlas/PAH_675093.h5ad'
# atlas_name = get_atlas_name(atlas_path)
# print('Read atlas')
# adata = read_atlas(atlas_path)
# print('Calc QC')
# figure_path = '/Users/christophebecavin/Documents/testatlas/'
# atlas.create_qc_plots(adata, atlas_path, atlas_name, figure_path)



def clust_analysis_external(adata, partition_key):
    """

    Parameters
    ----------
    adata
    partition_key

    Returns
    -------

    """


def clust_analysis_internal(adata, partition_key):
    """

    Parameters
    ----------
    adata
    partition_key

    Returns
    -------

    """



from anndata import AnnData
from sklearn.metrics import (
    adjusted_mutual_info_score,
    adjusted_rand_score,
    calinski_harabasz_score,
    davies_bouldin_score,
    fowlkes_mallows_score,
    silhouette_score,
    v_measure_score,
)
from sklearn.preprocessing import LabelEncoder


def silhouette(adata, partition_key, obsm_representation: str = None):
    """
    By default, computes the average silhouette coefficient for the adata
    with respect to the clustering specified by partition_key.
    If given a value for obsm_representation, computes the index on
    the representation of the original data stored at
    adata.obsm.obsm_representation.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotations to be
        used.

    obsm_representation : str
        Key of adata.obsm containing a representation of the data for
        example 'umap' or
        'pca' etc...

    Returns
    -------
    The the average silhouette coefficient of the data with respect to the
    partition_key clustering
    """
    annotations = adata.obs[partition_key]
    if obsm_representation:
        count_repr = adata.obsm[obsm_representation]
        return silhouette_score(count_repr, annotations)
    else:
        original_count = adata.X
        return silhouette_score(original_count, annotations)


# specify a "subset" parameter corresponding to a type to subset the adata.
# The metric would be computed on this subset. However, this requires to be
# rather careful with the used annotations etc... Maybe it would be smarter
# to create another module for subset analysis or at least a get_subset
# which computes correct fields for subsets. Just create a function
# which create a subset adata and would be used as an argument
# in the other function (use subset = 'subset_name')


def davies_bouldin(
    adata: AnnData, partition_key, obsm_representation: str = None
):
    """
    By default, computes the Davies-Bouldin index for the adata with respect
    to the clustering specified by partition_key.
    If given a value for obsm_representation, computes the index on the
    representation of the original data stored at
    adata.obsm.obsm_representation.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotations to be
        used.

    obsm_representation : str
        Key of adata.obsm containing a representation of the data for
        example 'umap' or
        'pca' etc...

    Returns
    -------
    The Davies-Bouldin index of the data with respect to the partition_key
    clustering
    """
    annotations = adata.obs[partition_key]
    if obsm_representation:
        count_repr = adata.obsm[obsm_representation]
        return davies_bouldin_score(count_repr, annotations)
    else:
        original_count = adata.X
        return davies_bouldin_score(original_count, annotations)


def calinski_harabasz(
    adata: AnnData, partition_key, obsm_representation: str = None
):
    """
    By default, computes the Davies-Bouldin index for the adata with respect
    to the clustering specified by partition_key.
    If given a value for obsm_representation, computes the index on the
    representation of the original data stored at
    adata.obsm.obsm_representation.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotations to be used.

    obsm_representation : str
        Key of adata.obsm containing a representation of the data for example
        'umap' or
        'pca' etc...

    Returns
    -------
    The Davies-Bouldin index of the data with respect to the partition_key
    clustering
    """
    annotations = adata.obs[partition_key]
    if obsm_representation:
        count_repr = adata.obsm[obsm_representation]
        return calinski_harabasz_score(count_repr, annotations)
    else:
        original_count = adata.X
        return calinski_harabasz_score(original_count, annotations)


def dbcv(adata, partition_key, obsm_representation: str = None):
    """
    By default, computes the DBCV index for the adata with respect to the
    clustering
    specified by partition_key.
    If given a value for obsm_representation, computes the index on the
    representation of the original data stored at
    adata.obsm.obsm_representation.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotations to be used.

    obsm_representation : str
        Key of adata.obsm containing a representation of the data for example
        'umap' or 'pca' etc...

    Returns
    -------
    The DBCV index of the data with respect to the partition_key clustering
    """
    annotations = adata.obs[partition_key]
    if obsm_representation:
        count_repr = adata.obsm[obsm_representation]
        return davies_bouldin_score(count_repr, annotations)
    else:
        original_count = adata.X
        return davies_bouldin_score(original_count, annotations)


def annotation_to_num(adata, partition_key, reference):
    """
    Transforms the annotations from categorical to numerical

    Parameters
    ----------
    adata
    partition_key
    reference

    Returns
    -------

    """
    print(partition_key)
    print(reference)
    annotation = adata.obs[partition_key].to_numpy()
    ref_annotation = adata.obs[reference].to_numpy()
    le = LabelEncoder()
    le.fit(annotation)
    annotation = le.transform(annotation)
    ref_annotation = le.transform(ref_annotation)
    return annotation, ref_annotation


def rand(adata, partition_key, reference):
    """
    By default, computes the Rand index for the adata with respect to the
    clustering specified by partition_key compared to the reference clustering.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotation to be used.

    reference : str
        The key in adata.obs corresponding to the reference annotation to be
        used.

    Returns
    -------
    The Rand index of the data with respect to the partition_key clustering
    compared
    to the reference clustering.
    """
    return adjusted_rand_score(
        *annotation_to_num(adata, partition_key, reference)
    )


def fowlkes_mallows(adata, partition_key, reference):
    """
    By default, computes the Fowlkes-Mallows score for the adata with respect
    to theclustering specified by partition_key compared to the
    reference clustering.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotation to be used.

    reference : str
        The key in adata.obs corresponding to the reference annotation to be
        used.

    Returns
    -------
    The Fowlkes-Mallows score of the data with respect to the partition_key
    clustering compared to the reference clustering.
    """
    return fowlkes_mallows_score(
        *annotation_to_num(adata, partition_key, reference)
    )


def nmi(adata, partition_key, reference):
    """
    By default, computes the Normalized Mutual Information for the adata with
    respect to the clustering specified by partition_key compared to the
    reference clustering.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotation to be used.

    reference : str
        The key in adata.obs corresponding to the reference annotation to be
        used.

    Returns
    -------
    The Normalized Mutual Information of the data with respect to the
    partition_key clustering compared to the reference clustering.
    """
    return adjusted_mutual_info_score(
        *annotation_to_num(adata, partition_key, reference)
    )


def vmeasure(adata, partition_key, reference):
    """
    By default, computes the V-Measure for the adata with respect to the
    clustering specified by partition_key compared to the reference
    clustering.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotation to be used.

    reference : str
        The key in adata.obs corresponding to the reference annotation to
        be used.

    Returns
    -------
    The V-Measure of the data with respect to the partition_key clustering
    compared to the reference clustering.
    """
    return v_measure_score(*annotation_to_num(adata, partition_key, reference))


def dunn(adata, partition_key: str, obsm_representation: str = None):
    """
    By default, computes the Dunn index for the adata with respect to the
    clustering specified by partition_key.
    If given a value for obsm_representation, computes the index on the
    representation of the original data stored at
    adata.obsm.obsm_representation.

    Parameters
    ----------
    adata : anndata
        The corrected expression matrix

    partition_key : str
        The key in adata.obs corresponding to the annotations to be used.

    obsm_representation : str
        Key of adata.obsm containing a representation of the data for
        example 'umap' or 'pca' etc...

    Returns
    -------
    The Dunn index of the data with respect to the partition_key clustering
    """



import argparse  # pragma: no cover

from . import checkatlas  # pragma: no cover


def main() -> None:  # pragma: no cover
    """
    The main function executes on commands:
    `python -m checkatlas` and `$ checkatlas `.

    This is your program's entry point.

    You can change this function to do whatever you want.
    Examples:
        * Run a test suite
        * Run a server
        * Do some other stuff
        * Run a command line application (Click, Typer, ArgParse)
        * List all available tasks
        * Run an application (Flask, FastAPI, Django, etc.)
    """
    parser = argparse.ArgumentParser(
        description="checkatlas.",
        epilog="Enjoy the checkatlas functionality!",
    )
    # This is required positional argument
    parser.add_argument(
        "path",
        type=str,
        help="Path containing Scanpy and Seurat atlases",
        default="becavin-lab",
    )
    # This is optional named argument
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Optionally adds verbosity",
    )
    args = parser.parse_args()
    print(f"Checking your single-cell atlases in {args.path}!")
    if args.verbose:
        print("Verbose mode is on.")

    print("Searching Seurat and Scanpy files")
    atlas_list = checkatlas.list_atlases(args.path)
    print(
        "Found",
        len(atlas_list),
        "files with these extensions",
        checkatlas.EXTENSIONS,
    )

    n_cpus = 4
    multithread = False
    checkatlas.run(args.path, atlas_list, multithread, n_cpus)


if __name__ == "__main__":  # pragma: no cover
    main()



import numpy as np
import pandas as pd
from scipy.spatial import distance_matrix


def euc(a, b):
    return np.sum(np.square(a - b))


# euclidian(np.array([1,2,3]),np.array([2,3,4]))


def kruskal_stress(adata, key_repr):
    """
    Computes the kruskal stress for the low dimension representation
    stored at adata.obsm.key_repr

    Parameters
    ----------
    adata
    key_repr

    Returns
    -------

    """
    for key in key_repr:
        print(key)
        high_dim_counts = adata.X
        low_dim_counts = adata.obsm[key]
        high_dim_distances = distance_matrix(high_dim_counts, high_dim_counts)
        low_dim_distances = distance_matrix(low_dim_counts, low_dim_counts)
        stress = np.sqrt(
            np.square(high_dim_distances - low_dim_distances).sum()
            / np.square(low_dim_distances).sum()
        )
        print(
            f"dist_diff = "
            f"{np.square(high_dim_distances - low_dim_distances).sum()}"
        )
        print(f"lowdimsum = {np.square(low_dim_distances).sum()}")
        df = pd.DataFrame(low_dim_distances)
        print(df.describe())
        print(stress)
    return stress


# kruskal_stress(test_set, key_repr=['X_pca','X_umap','X_tsne'])


def spearmans_rho(adata, key_repr):
    return



import pytest
import scanpy as sc

from checkatlas.atlas import clean_scanpy_atlas

given = pytest.mark.parametrize


def test_atlas_object():
    adata = sc.datasets.pbmc68k_reduced()
    atlas_info = ["PBMC68k", "Scanpy", ".h5ad", "Scanpy module"]
    assert clean_scanpy_atlas(adata, atlas_info)


# test_atlas_object()
# @given("fn", [atlas(), list_atlases()])
# def test_parameterized(fn):
#     assert "hello from" in fn()
#
#
# def test_list_atlases():
#     assert list_atlases(".") == "hello from base function"
#
#
# def test_atlas():
#     assert atlas() == "hello from BaseClass"



import anndata
import matplotlib.pyplot as plt
import numpy as np
import scanpy as sc
from scipy.spatial import distance_matrix
from sklearn.cluster import dbscan, k_means
from sklearn.preprocessing import LabelEncoder

from .clust_plots import hist_internal

adata = anndata.read_h5ad(
    r"C:\Users\ipmc\Documents\Package_Metrics\scanpy\scanpy\datasets\
    HCA_Barbry_Hg19_seurat.h5ad"
)
adata.X = adata.X.todense()

sc.pp.neighbors(adata)

sc.tl.leiden(adata=adata)
sc.tl.louvain(adata=adata)

sc.pl.umap(adata, color=["CellType"])

hist_internal(
    adata=adata, partition_keys=["leiden", "louvain"], reference=["CellType"]
)

le = LabelEncoder()
le.fit(adata.obs["CellType"])
classes = le.classes_
n_c = len(classes)
# f, axes = plt.subplots(4,6)
i = 1
for type in classes:
    category = adata[adata.obs["CellType"] == type]
    sc.pl.umap(category, ax=plt.subplot(4, 6, i))
    plt.title(type)
    i += 1
sc.pl.umap(adata, color=["CellType"])


kmeans = k_means(adata.X, 28)
np.array([str(x) for x in kmeans[1]])
dbscan = dbscan(adata.X, 28)
adata.obs["dbscan"] = dbscan[1]


sc.pl.umap(adata, color=["kmeans"])
sc.pp.pca(adata, n_comps=20)

# phenograph.cluster(data = adata.obsm['X_pca'][:,:12])

phen_typing = sc.external.tl.phenograph(adata.obsm["X_pca"][:, :12])

x = adata.X.todense()

dist = distance_matrix(x[:10, :], x[:10, :])

# Create test set
brain = anndata.read_h5ad(
    r"C:\Users\ipmc\Documents\Package_Metrics\scanpy\scanpy\
    datasets\brain.h5ad"
)
sc.pl.highest_expr_genes(
    brain,
    n_top=20,
)

sc.pp.filter_cells(brain, min_genes=200)
sc.pp.filter_genes(brain, min_cells=3)

sc.pp.normalize_total(brain, target_sum=1e4)
sc.pp.log1p(brain)

sc.tl.pca(brain, svd_solver="arpack")

sc.pp.neighbors(brain, n_neighbors=10, n_pcs=40)

sc.tl.umap(brain)

sc.tl.louvain(brain)
sc.tl.leiden(brain)

sc.pl.umap(brain, color=["leiden", "louvain"])
sc.tl.tsne(brain)

brain.write_h5ad(
    r"C:\Users\ipmc\Documents\Package_Metrics\scanpy\scanpy\datasets\
    test_dataset.h5ad"
)

# Load test set

test_set = anndata.read_h5ad(
    r"C:\Users\ipmc\Documents\Package_Metrics\scanpy\scanpy\datasets\
    test_dataset.h5ad"
)



import os
import re

import numpy as np
import pandas as pd
import scanpy as sc

from . import checkatlas, folders
from .metrics.cluster import clust_compute

# try:
#     from .metrics.cluster import clust_compute
# except ImportError:
#     from metrics.cluster import clust_compute
# try:
#     from .metrics.dim_red import dr_compute
# except ImportError:
#     from metrics.dim_red import dr_compute

# try:
#     from . import checkatlas, folders
# except ImportError:
#     import folders
#     import checkatlas


"""
Atlas module
All the function to screen the atlases
"""

OBS_CLUSTERS = [
    "cell_type",
    "CellType",
    "celltype",
    "ann_finest_level",
    "seurat_clusters",
    "louvain",
    "leiden",
    "orig.ident",
]


def list_atlases(path) -> list:
    """
    List all atlases files in the path
    Detect .rds, .h5, .h5ad
    :param path:
    :return: List of files
    """
    atlas_list = list()
    for root, dirs, files in os.walk(path):
        for file in files:
            for extension in checkatlas.EXTENSIONS:
                if file.endswith(extension):
                    atlas_list.append(os.path.join(root, file))
    return atlas_list


def convert_atlas(atlas_path, atlas_name) -> None:
    """
    Convert a atlas to Scanpy
    :param atlas_path:
    :return:
    """
    print("Convert Seurat object to Scanpy: ", atlas_path)
    rscript_cmd = (
        "Rscript "
        + checkatlas.RSCRIPT
        + " "
        + os.path.dirname(atlas_path)
        + " "
        + atlas_name
    )
    print(rscript_cmd)
    os.system(rscript_cmd)


def clean_scanpy_atlas(adata, atlas_info) -> bool:
    """
    Clean the Scanpy object to be suyre to get all information out of it
    :param adata:
    :return:
    """
    print("Clean scanpy:" + atlas_info[0])
    # If OBS_CLUSTERS are present and in int32 -> be sure to
    # transform them in categorical
    for obs_key in adata.obs_keys():
        for obs_key_celltype in OBS_CLUSTERS:
            if obs_key_celltype in obs_key:
                if adata.obs[obs_key].dtype == np.int32:
                    adata.obs[obs_key] = pd.Categorical(adata.obs[obs_key])
    return adata


def get_viable_obs(adata):
    """
    Search in obs_keys a match to OBS_CLUSTERS values
    Extract sorted obs_keys in same order then OBS_CLUSTERS
    :param adata:
    :return:
    """
    obs_keys = list()
    for obs_key in adata.obs_keys():
        for obs_key_celltype in OBS_CLUSTERS:
            if obs_key_celltype in obs_key:
                if type(adata.obs[obs_key].dtype) == pd.CategoricalDtype:
                    obs_keys.append(obs_key)
    # ### obs are sorted to have cell_type first
    # (! Need to fix that accordingly)
    return sorted(obs_keys)


def create_summary_table(adata, atlas_path, atlas_info, path) -> None:
    """
    Create a table with all interesting variables
    :param adata:
    :param atlas_name:
    :param csv_path:
    :return:
    """
    atlas_name = atlas_info[0]
    atlas_file_type = atlas_info[1]
    atlas_extension = atlas_info[2]
    csv_path = os.path.join(
        folders.get_folder(path, folders.SUMMARY),
        atlas_name + checkatlas.SUMMARY_EXTENSION,
    )
    # Create summary table
    header = [
        "AtlasFileType",
        "NbCells",
        "NbGenes",
        "AnnData.raw",
        "AnnData.X",
        "File_extension",
        "File_path",
    ]
    print("Run summary")
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)
    df_summary["AtlasFileType"][atlas_name] = atlas_file_type
    df_summary["NbCells"][atlas_name] = adata.n_obs
    df_summary["NbGenes"][atlas_name] = adata.n_vars
    df_summary["AnnData.raw"][atlas_name] = adata.raw is not None
    df_summary["AnnData.X"][atlas_name] = adata.X is not None
    df_summary["File_extension"][atlas_name] = atlas_extension
    df_summary["File_path"][atlas_name] = atlas_path.replace(path, "")
    df_summary.to_csv(csv_path, index=False)


def create_anndata_table(adata, atlas_path, atlas_info, path) -> None:
    """
    Create a table with all AnnData arguments
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    csv_path = os.path.join(
        folders.get_folder(path, folders.ANNDATA),
        atlas_name + checkatlas.ADATA_EXTENSION,
    )
    # Create AnnData table
    header = ["obs", "obsm", "var", "varm", "uns"]
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)
    # html_element = "<span class=\"label label-primary\">"
    # new_line = ''
    # for value in list(adata.obs.columns):
    #     new_line += html_element + value + "</span><br>"
    #     print(new_line)
    df_summary["obs"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.obs.columns))
        + "</code>"
    )
    df_summary["obsm"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.obsm_keys()))
        + "</code>"
    )
    df_summary["var"][atlas_name] = (
        "<code>" + "</code><br><code>".join(list(adata.var_keys())) + "</code>"
    )
    df_summary["varm"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.varm_keys()))
        + "</code>"
    )
    df_summary["uns"][atlas_name] = (
        "<code>" + "</code><br><code>".join(list(adata.uns_keys())) + "</code>"
    )
    df_summary.to_csv(csv_path, index=False, quoting=False)


def create_qc_plots(adata, atlas_path, atlas_info, path) -> None:
    """
    Display the atlas QC
    Search for the OBS variable which correspond to the toal_RNA, total_UMI,
     MT_ratio, RT_ratio
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    sc.settings.figdir = folders.get_workingdir(path)
    qc_path = os.sep + atlas_name + checkatlas.QC_EXTENSION
    print("Calc QC")
    # mitochondrial genes
    adata.var["mt"] = adata.var_names.str.startswith("MT-")
    # ribosomal genes
    adata.var["ribo"] = adata.var_names.str.startswith(("RPS", "RPL"))
    sc.pp.calculate_qc_metrics(
        adata,
        qc_vars=["mt", "ribo"],
        percent_top=None,
        log1p=False,
        inplace=True,
    )
    sc.pl.violin(
        adata,
        [
            "n_genes_by_counts",
            "total_counts",
            "pct_counts_mt",
            "pct_counts_ribo",
        ],
        jitter=0.4,
        multi_panel=True,
        show=False,
        save=qc_path,
    )


def create_umap_fig(adata, atlas_path, atlas_info, path) -> None:
    """
    Display the UMAP of celltypes
    Search for the OBS variable which correspond to the celltype annotation
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    # Search if tsne reduction exists
    r = re.compile(".*umap*.")
    print(len(list(filter(r.match, adata.obsm_keys()))))
    if len(list(filter(r.match, adata.obsm_keys()))) > 0:
        # Setting up figures directory
        sc.settings.figdir = folders.get_workingdir(path)
        umap_path = os.sep + atlas_name + checkatlas.UMAP_EXTENSION
        # Exporting umap
        obs_keys = get_viable_obs(adata)
        if len(obs_keys) != 0:
            sc.pl.umap(adata, color=obs_keys[0], show=False, save=umap_path)
        else:
            sc.pl.umap(adata, show=False, save=umap_path)


def create_tsne_fig(adata, atlas_path, atlas_info, path) -> None:
    """
    Display the TSNE of celltypes
    Search for the OBS variable which correspond to the celltype annotation
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    # Search if tsne reduction exists
    atlas_name = atlas_info[0]
    r = re.compile(".*tsne*.")
    if len(list(filter(r.match, adata.obsm_keys()))) > 0:
        # Setting up figures directory
        sc.settings.figdir = sc.settings.figdir = folders.get_workingdir(path)
        tsne_path = os.sep + atlas_name + checkatlas.TSNE_EXTENSION
        # Exporting tsne
        obs_keys = get_viable_obs(adata)
        if len(obs_keys) != 0:
            sc.pl.tsne(adata, color=obs_keys[0], show=False, save=tsne_path)
        else:
            sc.pl.tsne(adata, show=False, save=tsne_path)


def metric_cluster(adata, atlas_path, atlas_info, path) -> None:
    """
    Main function of checkatlas
    For every atlas create summary tables with all attributes of the atlas
    Calc UMAP, tSNE, andd all metrics
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    csv_path = os.path.join(
        folders.get_folder(path, folders.CLUSTER),
        atlas_name + checkatlas.METRIC_CLUSTER_EXTENSION,
    )
    header = ["Sample", "obs", "Silhouette", "Davies-Bouldin"]
    df_cluster = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs(adata)
    for obs_key in obs_keys:
        # Need more than one sample to calculate metric
        annotations = adata.obs[obs_key]
        if len(annotations.cat.categories) != 1:
            silhouette = 1
            print("Calc Silhouette for " + atlas_name, obs_key)
            silhouette = clust_compute.silhouette(adata, obs_key, "X_umap")
            daviesb = -1
            print("Calc Davies Bouldin for " + atlas_name, obs_key)
            daviesb = clust_compute.davies_bouldin(adata, obs_key, "X_umap")
            df_line = pd.DataFrame(
                {
                    "Sample": [atlas_name + "_" + obs_key],
                    "obs": [obs_key],
                    "Silhouette": [silhouette],
                    "Davies-Bouldin": [daviesb],
                }
            )
            df_cluster = pd.concat(
                [df_cluster, df_line], ignore_index=True, axis=0
            )
    if len(df_cluster) != 0:
        df_cluster.to_csv(csv_path, index=False)


def metric_annot(adata, atlas_path, atlas_info, path) -> None:
    """
    Main function of checkatlas
    For every atlas create summary tables with all attributes of the atlas
    Calc UMAP, tSNE, and all metrics
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    csv_path = os.path.join(
        folders.get_folder(path, folders.ANNOTATION),
        atlas_name + checkatlas.METRIC_ANNOTATION_EXTENSION,
    )
    header = ["Sample", "obs", "Rand"]
    df_annot = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs(adata)
    if len(obs_keys) != 0:
        ref_obs = obs_keys[0]
        for i in range(1, len(obs_keys)):
            obs_key = obs_keys[i]
            # Need more than one sample to calculate metric
            annotations = adata.obs[obs_key]
            if len(annotations.cat.categories) != 1:
                print(
                    "NOT WORKING YET - Calc Rand Index for " + atlas_name,
                    obs_key,
                )
                rand = -1
                # ##rand = clust_compute.rand(adata, obs_key, ref_obs)
                df_line = pd.DataFrame(
                    {
                        "Sample": [atlas_name + "_" + obs_key],
                        "Reference": [ref_obs],
                        "obs": [obs_key],
                        "Rand": [rand],
                    }
                )
                df_annot = pd.concat(
                    [df_annot, df_line], ignore_index=True, axis=0
                )
        if len(df_annot) != 0:
            df_annot.to_csv(csv_path, index=False)


def metric_dimred(adata, atlas_path, atlas_info, path) -> None:
    """
    Main function of checkatlas
    For every atlas create summary tables with all attributes of the atlas
    Calc UMAP, tSNE, andd all metrics
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[0]
    csv_path = os.path.join(
        folders.get_folder(path, folders.DIMRED),
        atlas_name + checkatlas.METRIC_DIMRED_EXTENSION,
    )
    header = ["Sample", "obs", "Kruskal"]
    df_dimred = pd.DataFrame(columns=header)
    for obsm_key in adata.obsm_keys():
        print(
            "NOT WORKING YET - Calc Kruskal Stress for " + atlas_name, obsm_key
        )
        kruskal = -1
        # kruskal = dr_compute.kruskal_stress(adata, obsm_key)
        df_line = pd.DataFrame(
            {
                "Sample": [atlas_name + "_" + obsm_key],
                "obs": [obsm_key],
                "Kruskal": [kruskal],
            }
        )
        df_dimred = pd.concat([df_dimred, df_line], ignore_index=True, axis=0)
    if len(df_dimred) != 0:
        df_dimred.to_csv(csv_path, index=False)



import os


def run_multiqc(path):
    title = '"Check Single-Cell Atlas"'
    name = "CheckAtlas"
    multiqc_script = (
        "cd " + path + ";"
        "multiqc -i " + title + " -n " + name + " -o CheckAtlas_MultiQC"
        ' -f -e "snippy" --cl-config "ignore_images: false" ' + path
    )
    print(multiqc_script)
    os.system(multiqc_script)
